"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _errors = require("./errors");

var common = _interopRequireWildcard(require("./common"));

var keys = _interopRequireWildcard(require("./keys"));

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

var convertAddChange = function convertAddChange(change) {
  switch (change.type) {
    case 'account':
      return {
        type: 'put',
        key: keys.typeKeyToSerializeKey.account(change.value),
        value: change.value.serializeWire()
      };

    case 'action':
      return {
        type: 'put',
        key: keys.typeKeyToSerializeKey.action({
          blockIndex: change.value.blockIndex,
          transactionIndex: change.value.transactionIndex,
          index: change.value.index
        }),
        value: change.value.serializeWire()
      };

    case 'asset':
      return {
        type: 'put',
        key: keys.typeKeyToSerializeKey.asset(change.value),
        value: change.value.serializeWire()
      };

    case 'block':
      return [{
        type: 'put',
        key: keys.typeKeyToSerializeKey.block(change.value),
        value: change.value.serializeWire()
      }, {
        type: 'put',
        key: keys.maxBlockHashKey,
        value: common.serializeBlockHash(change.value.hash)
      }];

    case 'blockSystemFee':
      return {
        type: 'put',
        key: keys.typeKeyToSerializeKey.blockSystemFee(change.value),
        value: change.value.serializeWire()
      };

    case 'header':
      return [{
        type: 'put',
        key: keys.typeKeyToSerializeKey.header(change.value),
        value: change.value.serializeWire()
      }, {
        type: 'put',
        key: keys.maxHeaderHashKey,
        value: common.serializeHeaderHash(change.value.hash)
      }, {
        type: 'put',
        key: keys.serializeHeaderIndexHashKey(change.value.index),
        value: common.serializeHeaderHash(change.value.hash)
      }];

    case 'transaction':
      return {
        type: 'put',
        key: keys.typeKeyToSerializeKey.transaction(change.value),
        value: change.value.serializeWire()
      };

    case 'transactionSpentCoins':
      return {
        type: 'put',
        key: keys.typeKeyToSerializeKey.transactionSpentCoins(change.value),
        value: change.value.serializeWire()
      };

    case 'contract':
      return {
        type: 'put',
        key: keys.typeKeyToSerializeKey.contract(change.value),
        value: change.value.serializeWire()
      };

    case 'storageItem':
      return {
        type: 'put',
        key: keys.typeKeyToSerializeKey.storageItem({
          hash: change.value.hash,
          key: change.value.key
        }),
        value: change.value.serializeWire()
      };

    case 'validator':
      return {
        type: 'put',
        key: keys.typeKeyToSerializeKey.validator({
          publicKey: change.value.publicKey
        }),
        value: change.value.serializeWire()
      };

    case 'invocationData':
      return {
        type: 'put',
        key: keys.typeKeyToSerializeKey.invocationData(change.value),
        value: change.value.serializeWire()
      };

    default:
      // eslint-disable-next-line
      change.type;
      throw new _errors.UnknownTypeError();
  }
};

var convertDeleteChange = function convertDeleteChange(change) {
  switch (change.type) {
    case 'account':
      return {
        type: 'del',
        key: keys.typeKeyToSerializeKey.account(change.key)
      };

    case 'contract':
      return {
        type: 'del',
        key: keys.typeKeyToSerializeKey.contract(change.key)
      };

    case 'storageItem':
      return {
        type: 'del',
        key: keys.typeKeyToSerializeKey.storageItem(change.key)
      };

    default:
      // eslint-disable-next-line
      change.type;
      throw new _errors.UnknownTypeError();
  }
};

var _default = function _default(change) {
  if (change.type === 'add') {
    var result = convertAddChange(change.change);
    ;
    return Array.isArray(result) ? result : [result];
  } else if (change.type === 'delete') {
    return [convertDeleteChange(change.change)];
  } // eslint-disable-next-line


  change.type; // TODO: Make better

  throw new Error('Bad change type');
};

exports.default = _default;