"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.typeKeyToSerializeKeyString = exports.typeKeyToSerializeKey = exports.serializeOutputKeyString = exports.accountMaxKey = exports.accountMinKey = exports.validatorMaxKey = exports.validatorMinKey = exports.getActionKeyMax = exports.getActionKeyMin = exports.serializeActionKeyString = exports.serializeActionKey = exports.getStorageItemKeyMax = exports.getStorageItemKeyMin = exports.maxBlockHashKey = exports.maxHeaderHashKey = exports.serializeHeaderIndexHashKeyString = exports.serializeHeaderIndexHashKey = void 0;

var _neoBlockchainCore = require("neo-blockchain-core");

var _bytewise = _interopRequireDefault(require("bytewise"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var accountKeyPrefix = 'account';
var actionKeyPrefix = 'action';
var assetKeyPrefix = 'asset';
var blockKeyPrefix = 'block';
var blockSystemFeeKeyPrefix = 'blockSystemFee';
var headerKeyPrefix = 'header';
var headerHashKeyPrefix = 'header-index';
var transactionKeyPrefix = 'transaction';
var transactionSpentCoinsKeyPrefix = 'transactionSpentCoins';
var contractKeyPrefix = 'contract';
var storageItemKeyPrefix = 'storageItem';
var validatorKeyPrefix = 'validator';
var invocationDataKeyPrefix = 'invocationData';
var settingsPrefix = 'settings';

var serializeHeaderIndexHashKey = function serializeHeaderIndexHashKey(index) {
  return _bytewise.default.encode([headerHashKeyPrefix, index]);
};

exports.serializeHeaderIndexHashKey = serializeHeaderIndexHashKey;

var serializeHeaderIndexHashKeyString = function serializeHeaderIndexHashKeyString(index) {
  return "".concat(headerHashKeyPrefix, ":") + "".concat(index);
};

exports.serializeHeaderIndexHashKeyString = serializeHeaderIndexHashKeyString;

var maxHeaderHashKey = _bytewise.default.encode([settingsPrefix, 'max-header-hash']);

exports.maxHeaderHashKey = maxHeaderHashKey;

var maxBlockHashKey = _bytewise.default.encode([settingsPrefix, 'max-block-hash']);

exports.maxBlockHashKey = maxBlockHashKey;

var serializeStorageItemKey = function serializeStorageItemKey(_ref) {
  var hash = _ref.hash,
      key = _ref.key;
  return _bytewise.default.encode([storageItemKeyPrefix, _neoBlockchainCore.common.uInt160ToBuffer(hash), key]);
};

var serializeStorageItemKeyString = function serializeStorageItemKeyString(_ref2) {
  var hash = _ref2.hash,
      key = _ref2.key;
  return "".concat(storageItemKeyPrefix, ":") + "".concat(_neoBlockchainCore.common.uInt160ToString(hash), ":") + "".concat(key.toString('hex'));
};

var getStorageItemKeyMin = function getStorageItemKeyMin(_ref3) {
  var hash = _ref3.hash;
  return _bytewise.default.encode(hash == null ? _bytewise.default.sorts.array.bound.lower([storageItemKeyPrefix]) : _bytewise.default.sorts.array.bound.lower([storageItemKeyPrefix, _neoBlockchainCore.common.uInt160ToBuffer(hash)]));
};

exports.getStorageItemKeyMin = getStorageItemKeyMin;

var getStorageItemKeyMax = function getStorageItemKeyMax(_ref4) {
  var hash = _ref4.hash;
  return _bytewise.default.encode(hash == null ? _bytewise.default.sorts.array.bound.upper([storageItemKeyPrefix]) : _bytewise.default.sorts.array.bound.upper([storageItemKeyPrefix, _neoBlockchainCore.common.uInt160ToBuffer(hash)]));
};

exports.getStorageItemKeyMax = getStorageItemKeyMax;

var serializeActionKey = function serializeActionKey(_ref5) {
  var blockIndex = _ref5.blockIndex,
      transactionIndex = _ref5.transactionIndex,
      index = _ref5.index;
  return _bytewise.default.encode([actionKeyPrefix, blockIndex, transactionIndex, index]);
};

exports.serializeActionKey = serializeActionKey;

var serializeActionKeyString = function serializeActionKeyString(_ref6) {
  var blockIndex = _ref6.blockIndex,
      transactionIndex = _ref6.transactionIndex,
      index = _ref6.index;
  return "".concat(actionKeyPrefix, ":") + "".concat(blockIndex, ":") + "".concat(transactionIndex, ":") + "".concat(index);
};

exports.serializeActionKeyString = serializeActionKeyString;

var getActionKeyMin = function getActionKeyMin(_ref7) {
  var blockIndexStart = _ref7.blockIndexStart,
      transactionIndexStart = _ref7.transactionIndexStart,
      indexStart = _ref7.indexStart;
  return _bytewise.default.encode(_bytewise.default.sorts.array.bound.lower([actionKeyPrefix, blockIndexStart, transactionIndexStart, indexStart].filter(function (value) {
    return value != null;
  })));
};

exports.getActionKeyMin = getActionKeyMin;

var getActionKeyMax = function getActionKeyMax(_ref8) {
  var blockIndexStop = _ref8.blockIndexStop,
      transactionIndexStop = _ref8.transactionIndexStop,
      indexStop = _ref8.indexStop;
  return _bytewise.default.encode(_bytewise.default.sorts.array.bound.upper([actionKeyPrefix, blockIndexStop == null ? Number.MAX_SAFE_INTEGER : blockIndexStop, transactionIndexStop == null ? Number.MAX_SAFE_INTEGER : transactionIndexStop, indexStop == null ? Number.MAX_SAFE_INTEGER : indexStop]));
};

exports.getActionKeyMax = getActionKeyMax;

var serializeValidatorKey = function serializeValidatorKey(_ref9) {
  var publicKey = _ref9.publicKey;
  return _bytewise.default.encode([validatorKeyPrefix, _neoBlockchainCore.common.ecPointToBuffer(publicKey)]);
};

var serializeValidatorKeyString = function serializeValidatorKeyString(_ref10) {
  var publicKey = _ref10.publicKey;
  return "".concat(validatorKeyPrefix, ":").concat(_neoBlockchainCore.common.ecPointToString(publicKey));
};

var validatorMinKey = _bytewise.default.encode(_bytewise.default.sorts.array.bound.lower([validatorKeyPrefix]));

exports.validatorMinKey = validatorMinKey;

var validatorMaxKey = _bytewise.default.encode(_bytewise.default.sorts.array.bound.upper([validatorKeyPrefix]));

exports.validatorMaxKey = validatorMaxKey;

var serializeUInt160Key = function serializeUInt160Key(_ref11) {
  var hash = _ref11.hash;
  return _neoBlockchainCore.common.uInt160ToBuffer(hash);
};

var serializeUInt256Key = function serializeUInt256Key(_ref12) {
  var hash = _ref12.hash;
  return _neoBlockchainCore.common.uInt256ToBuffer(hash);
};

var createSerializeUInt160Key = function createSerializeUInt160Key(prefix) {
  return function (input) {
    return _bytewise.default.encode([prefix, serializeUInt160Key(input)]);
  };
};

var createSerializeUInt256Key = function createSerializeUInt256Key(prefix) {
  return function (input) {
    return _bytewise.default.encode([prefix, serializeUInt256Key(input)]);
  };
};

var createSerializeUInt160KeyString = function createSerializeUInt160KeyString(prefix) {
  return function (input) {
    return "".concat(prefix, ":").concat(_neoBlockchainCore.common.uInt160ToString(input.hash));
  };
};

var createSerializeUInt256KeyString = function createSerializeUInt256KeyString(prefix) {
  return function (input) {
    return "".concat(prefix, ":").concat(_neoBlockchainCore.common.uInt256ToString(input.hash));
  };
};

var accountMinKey = _bytewise.default.encode(_bytewise.default.sorts.array.bound.lower([accountKeyPrefix]));

exports.accountMinKey = accountMinKey;

var accountMaxKey = _bytewise.default.encode(_bytewise.default.sorts.array.bound.upper([accountKeyPrefix]));

exports.accountMaxKey = accountMaxKey;

var serializeOutputKeyString = function serializeOutputKeyString(_ref13) {
  var index = _ref13.index,
      hash = _ref13.hash;
  return "".concat(_neoBlockchainCore.common.uInt256ToString(hash), ":").concat(index);
};

exports.serializeOutputKeyString = serializeOutputKeyString;
var typeKeyToSerializeKey = {
  account: createSerializeUInt160Key(accountKeyPrefix),
  action: serializeActionKey,
  asset: createSerializeUInt256Key(assetKeyPrefix),
  block: createSerializeUInt256Key(blockKeyPrefix),
  blockSystemFee: createSerializeUInt256Key(blockSystemFeeKeyPrefix),
  header: createSerializeUInt256Key(headerKeyPrefix),
  transaction: createSerializeUInt256Key(transactionKeyPrefix),
  transactionSpentCoins: createSerializeUInt256Key(transactionSpentCoinsKeyPrefix),
  contract: createSerializeUInt160Key(contractKeyPrefix),
  storageItem: serializeStorageItemKey,
  validator: serializeValidatorKey,
  invocationData: createSerializeUInt256Key(invocationDataKeyPrefix)
};
exports.typeKeyToSerializeKey = typeKeyToSerializeKey;
var typeKeyToSerializeKeyString = {
  account: createSerializeUInt160KeyString(accountKeyPrefix),
  action: serializeActionKeyString,
  asset: createSerializeUInt256KeyString(assetKeyPrefix),
  block: createSerializeUInt256KeyString(blockKeyPrefix),
  blockSystemFee: createSerializeUInt256KeyString(blockSystemFeeKeyPrefix),
  header: createSerializeUInt256KeyString(headerKeyPrefix),
  transaction: createSerializeUInt256KeyString(transactionKeyPrefix),
  transactionSpentCoins: createSerializeUInt256KeyString(transactionSpentCoinsKeyPrefix),
  contract: createSerializeUInt160KeyString(contractKeyPrefix),
  storageItem: serializeStorageItemKeyString,
  validator: serializeValidatorKeyString,
  invocationData: createSerializeUInt256KeyString(invocationDataKeyPrefix)
};
exports.typeKeyToSerializeKeyString = typeKeyToSerializeKeyString;