"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _neoBlockchainCore = require("neo-blockchain-core");

var _neoBlockchainNodeCore = require("neo-blockchain-node-core");

var _errors = require("./errors");

var common = _interopRequireWildcard(require("./common"));

var _convertChange = _interopRequireDefault(require("./convertChange"));

var keys = _interopRequireWildcard(require("./keys"));

var read = _interopRequireWildcard(require("./read"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } } function _next(value) { step("next", value); } function _throw(err) { step("throw", err); } _next(); }); }; }

var _default = function _default(_ref) {
  var db = _ref.db,
      context = _ref.context;

  var getHash = function () {
    var _ref3 = _asyncToGenerator(
    /*#__PURE__*/
    regeneratorRuntime.mark(function _callee(_ref2) {
      var hashOrIndex, hash, result;
      return regeneratorRuntime.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              hashOrIndex = _ref2.hashOrIndex;
              hash = hashOrIndex;

              if (!(typeof hash === 'number')) {
                _context.next = 15;
                break;
              }

              _context.prev = 3;
              _context.next = 6;
              return db.get(keys.serializeHeaderIndexHashKey(hash));

            case 6:
              result = _context.sent;
              hash = common.deserializeHeaderHash(result);
              _context.next = 15;
              break;

            case 10:
              _context.prev = 10;
              _context.t0 = _context["catch"](3);

              if (!_context.t0.notFound) {
                _context.next = 14;
                break;
              }

              throw new _errors.KeyNotFoundError("".concat(hash));

            case 14:
              throw _context.t0;

            case 15:
              return _context.abrupt("return", hash);

            case 16:
            case "end":
              return _context.stop();
          }
        }
      }, _callee, this, [[3, 10]]);
    }));

    return function getHash(_x) {
      return _ref3.apply(this, arguments);
    };
  }();

  var headerBase = read.createReadStorage({
    db: db,
    serializeKey: keys.typeKeyToSerializeKey.header,
    serializeKeyString: keys.typeKeyToSerializeKeyString.header,
    deserializeValue: function deserializeValue(buffer) {
      return _neoBlockchainCore.Header.deserializeWire({
        context: context,
        buffer: buffer
      });
    }
  });

  var getHeader = function () {
    var _ref5 = _asyncToGenerator(
    /*#__PURE__*/
    regeneratorRuntime.mark(function _callee2(_ref4) {
      var hashOrIndex, hash;
      return regeneratorRuntime.wrap(function _callee2$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              hashOrIndex = _ref4.hashOrIndex;
              _context2.next = 3;
              return getHash({
                hashOrIndex: hashOrIndex
              });

            case 3:
              hash = _context2.sent;
              return _context2.abrupt("return", headerBase.get({
                hash: hash
              }));

            case 5:
            case "end":
              return _context2.stop();
          }
        }
      }, _callee2, this);
    }));

    return function getHeader(_x2) {
      return _ref5.apply(this, arguments);
    };
  }();

  var header = {
    get: getHeader,
    tryGet: read.createTryGet({
      get: getHeader
    }),
    tryGetLatest: read.createTryGetLatest({
      db: db,
      latestKey: keys.maxHeaderHashKey,
      deserializeResult: function deserializeResult(result) {
        return {
          hash: common.deserializeHeaderHash(result)
        };
      },
      get: headerBase.get
    })
  };
  var blockBase = read.createReadStorage({
    db: db,
    serializeKey: keys.typeKeyToSerializeKey.block,
    serializeKeyString: keys.typeKeyToSerializeKeyString.block,
    deserializeValue: function deserializeValue(buffer) {
      return _neoBlockchainCore.Block.deserializeWire({
        context: context,
        buffer: buffer
      });
    }
  });

  var getBlock = function () {
    var _ref7 = _asyncToGenerator(
    /*#__PURE__*/
    regeneratorRuntime.mark(function _callee3(_ref6) {
      var hashOrIndex, hash;
      return regeneratorRuntime.wrap(function _callee3$(_context3) {
        while (1) {
          switch (_context3.prev = _context3.next) {
            case 0:
              hashOrIndex = _ref6.hashOrIndex;
              _context3.next = 3;
              return getHash({
                hashOrIndex: hashOrIndex
              });

            case 3:
              hash = _context3.sent;
              return _context3.abrupt("return", blockBase.get({
                hash: hash
              }));

            case 5:
            case "end":
              return _context3.stop();
          }
        }
      }, _callee3, this);
    }));

    return function getBlock(_x3) {
      return _ref7.apply(this, arguments);
    };
  }();

  var block = {
    get: getBlock,
    tryGet: read.createTryGet({
      get: getBlock
    }),
    tryGetLatest: read.createTryGetLatest({
      db: db,
      latestKey: keys.maxBlockHashKey,
      deserializeResult: function deserializeResult(result) {
        return {
          hash: common.deserializeBlockHash(result)
        };
      },
      get: blockBase.get
    })
  };
  var transaction = read.createReadStorage({
    db: db,
    serializeKey: keys.typeKeyToSerializeKey.transaction,
    serializeKeyString: keys.typeKeyToSerializeKeyString.transaction,
    deserializeValue: function deserializeValue(buffer) {
      return (0, _neoBlockchainCore.deserializeTransactionWire)({
        context: context,
        buffer: buffer
      });
    }
  });

  var getOutput = function () {
    var _ref9 = _asyncToGenerator(
    /*#__PURE__*/
    regeneratorRuntime.mark(function _callee4(_ref8) {
      var index, hash, tx, output;
      return regeneratorRuntime.wrap(function _callee4$(_context4) {
        while (1) {
          switch (_context4.prev = _context4.next) {
            case 0:
              index = _ref8.index, hash = _ref8.hash;
              _context4.next = 3;
              return transaction.get({
                hash: hash
              });

            case 3:
              tx = _context4.sent;
              output = tx.outputs[index];

              if (!(output == null)) {
                _context4.next = 7;
                break;
              }

              throw new _errors.KeyNotFoundError(keys.serializeOutputKeyString({
                index: index,
                hash: hash
              }));

            case 7:
              return _context4.abrupt("return", output);

            case 8:
            case "end":
              return _context4.stop();
          }
        }
      }, _callee4, this);
    }));

    return function getOutput(_x4) {
      return _ref9.apply(this, arguments);
    };
  }();

  var output = {
    get: getOutput,
    tryGet: read.createTryGet({
      get: getOutput
    })
  };
  return {
    header: header,
    block: block,
    blockSystemFee: read.createReadStorage({
      db: db,
      serializeKey: keys.typeKeyToSerializeKey.blockSystemFee,
      serializeKeyString: keys.typeKeyToSerializeKeyString.blockSystemFee,
      deserializeValue: function deserializeValue(buffer) {
        return _neoBlockchainNodeCore.BlockSystemFee.deserializeWire({
          context: context,
          buffer: buffer
        });
      }
    }),
    account: read.createReadAllStorage({
      db: db,
      serializeKey: keys.typeKeyToSerializeKey.account,
      serializeKeyString: keys.typeKeyToSerializeKeyString.account,
      minKey: keys.accountMinKey,
      maxKey: keys.accountMaxKey,
      deserializeValue: function deserializeValue(buffer) {
        return _neoBlockchainCore.Account.deserializeWire({
          context: context,
          buffer: buffer
        });
      }
    }),
    action: read.createReadGetAllStorage({
      db: db,
      serializeKey: keys.typeKeyToSerializeKey.action,
      serializeKeyString: keys.typeKeyToSerializeKeyString.action,
      getMinKey: keys.getActionKeyMin,
      getMaxKey: keys.getActionKeyMax,
      deserializeValue: function deserializeValue(buffer) {
        return (0, _neoBlockchainCore.deserializeActionWire)({
          context: context,
          buffer: buffer
        });
      }
    }),
    asset: read.createReadStorage({
      db: db,
      serializeKey: keys.typeKeyToSerializeKey.asset,
      serializeKeyString: keys.typeKeyToSerializeKeyString.asset,
      deserializeValue: function deserializeValue(buffer) {
        return _neoBlockchainCore.Asset.deserializeWire({
          context: context,
          buffer: buffer
        });
      }
    }),
    transaction: transaction,
    transactionSpentCoins: read.createReadStorage({
      db: db,
      serializeKey: keys.typeKeyToSerializeKey.transactionSpentCoins,
      serializeKeyString: keys.typeKeyToSerializeKeyString.transactionSpentCoins,
      deserializeValue: function deserializeValue(buffer) {
        return _neoBlockchainNodeCore.TransactionSpentCoins.deserializeWire({
          context: context,
          buffer: buffer
        });
      }
    }),
    output: output,
    contract: read.createReadStorage({
      db: db,
      serializeKey: keys.typeKeyToSerializeKey.contract,
      serializeKeyString: keys.typeKeyToSerializeKeyString.contract,
      deserializeValue: function deserializeValue(buffer) {
        return _neoBlockchainCore.Contract.deserializeWire({
          context: context,
          buffer: buffer
        });
      }
    }),
    storageItem: read.createReadGetAllStorage({
      db: db,
      serializeKey: keys.typeKeyToSerializeKey.storageItem,
      serializeKeyString: keys.typeKeyToSerializeKeyString.storageItem,
      getMinKey: keys.getStorageItemKeyMin,
      getMaxKey: keys.getStorageItemKeyMax,
      deserializeValue: function deserializeValue(buffer) {
        return _neoBlockchainCore.StorageItem.deserializeWire({
          context: context,
          buffer: buffer
        });
      }
    }),
    validator: read.createReadAllStorage({
      db: db,
      serializeKey: keys.typeKeyToSerializeKey.validator,
      serializeKeyString: keys.typeKeyToSerializeKeyString.validator,
      minKey: keys.validatorMinKey,
      maxKey: keys.validatorMaxKey,
      deserializeValue: function deserializeValue(buffer) {
        return _neoBlockchainCore.Validator.deserializeWire({
          context: context,
          buffer: buffer
        });
      }
    }),
    invocationData: read.createReadStorage({
      db: db,
      serializeKey: keys.typeKeyToSerializeKey.invocationData,
      serializeKeyString: keys.typeKeyToSerializeKeyString.invocationData,
      deserializeValue: function deserializeValue(buffer) {
        return _neoBlockchainCore.InvocationData.deserializeWire({
          context: context,
          buffer: buffer
        });
      }
    }),
    close: function () {
      var _ref10 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee5() {
        return regeneratorRuntime.wrap(function _callee5$(_context5) {
          while (1) {
            switch (_context5.prev = _context5.next) {
              case 0:
                _context5.next = 2;
                return db.close();

              case 2:
              case "end":
                return _context5.stop();
            }
          }
        }, _callee5, this);
      }));

      function close() {
        return _ref10.apply(this, arguments);
      }

      return close;
    }(),
    commit: function () {
      var _ref11 = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee6(changeSet) {
        var changes;
        return regeneratorRuntime.wrap(function _callee6$(_context6) {
          while (1) {
            switch (_context6.prev = _context6.next) {
              case 0:
                changes = changeSet.map(function (change) {
                  return (0, _convertChange.default)(change);
                }).reduce(function (acc, converted) {
                  return acc.concat(converted);
                }, []);
                _context6.next = 3;
                return db.batch(changes);

              case 3:
              case "end":
                return _context6.stop();
            }
          }
        }, _callee6, this);
      }));

      function commit(_x5) {
        return _ref11.apply(this, arguments);
      }

      return commit;
    }()
  };
};

exports.default = _default;