"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createTryGet = createTryGet;
exports.createTryGetLatest = createTryGetLatest;
exports.createReadStorage = createReadStorage;
exports.createAll = createAll;
exports.createReadAllStorage = createReadAllStorage;
exports.createReadGetAllStorage = createReadGetAllStorage;

var _errors = require("./errors");

var _streamToObservable = _interopRequireDefault(require("./streamToObservable"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } } function _next(value) { step("next", value); } function _throw(err) { step("throw", err); } _next(); }); }; }

function createTryGet(_ref) {
  var get = _ref.get;
  return function () {
    var _ref2 = _asyncToGenerator(
    /*#__PURE__*/
    regeneratorRuntime.mark(function _callee(key) {
      var result;
      return regeneratorRuntime.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              _context.prev = 0;
              _context.next = 3;
              return get(key);

            case 3:
              result = _context.sent;
              return _context.abrupt("return", result);

            case 7:
              _context.prev = 7;
              _context.t0 = _context["catch"](0);

              if (!_context.t0.notFound) {
                _context.next = 11;
                break;
              }

              return _context.abrupt("return", null);

            case 11:
              throw _context.t0;

            case 12:
            case "end":
              return _context.stop();
          }
        }
      }, _callee, this, [[0, 7]]);
    }));

    return function (_x) {
      return _ref2.apply(this, arguments);
    };
  }();
}

;

function createTryGetLatest(_ref3) {
  var db = _ref3.db,
      latestKey = _ref3.latestKey,
      deserializeResult = _ref3.deserializeResult,
      get = _ref3.get;
  return _asyncToGenerator(
  /*#__PURE__*/
  regeneratorRuntime.mark(function _callee2() {
    var result, _value;

    return regeneratorRuntime.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            _context2.prev = 0;
            _context2.next = 3;
            return db.get(latestKey);

          case 3:
            result = _context2.sent;
            _context2.next = 6;
            return get(deserializeResult(result));

          case 6:
            _value = _context2.sent;
            return _context2.abrupt("return", _value);

          case 10:
            _context2.prev = 10;
            _context2.t0 = _context2["catch"](0);

            if (!_context2.t0.notFound) {
              _context2.next = 14;
              break;
            }

            return _context2.abrupt("return", null);

          case 14:
            throw _context2.t0;

          case 15:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2, this, [[0, 10]]);
  }));
}

;

function createReadStorage(_ref5) {
  var db = _ref5.db,
      serializeKey = _ref5.serializeKey,
      serializeKeyString = _ref5.serializeKeyString,
      deserializeValue = _ref5.deserializeValue;

  var get = function () {
    var _ref6 = _asyncToGenerator(
    /*#__PURE__*/
    regeneratorRuntime.mark(function _callee3(key) {
      var result;
      return regeneratorRuntime.wrap(function _callee3$(_context3) {
        while (1) {
          switch (_context3.prev = _context3.next) {
            case 0:
              _context3.prev = 0;
              _context3.next = 3;
              return db.get(serializeKey(key));

            case 3:
              result = _context3.sent;
              return _context3.abrupt("return", deserializeValue(result));

            case 7:
              _context3.prev = 7;
              _context3.t0 = _context3["catch"](0);

              if (!_context3.t0.notFound) {
                _context3.next = 11;
                break;
              }

              throw new _errors.KeyNotFoundError(serializeKeyString(key));

            case 11:
              throw _context3.t0;

            case 12:
            case "end":
              return _context3.stop();
          }
        }
      }, _callee3, this, [[0, 7]]);
    }));

    return function get(_x2) {
      return _ref6.apply(this, arguments);
    };
  }();

  return {
    get: get,
    tryGet: createTryGet({
      get: get
    })
  };
}

function createAll(_ref7) {
  var db = _ref7.db,
      minKey = _ref7.minKey,
      maxKey = _ref7.maxKey,
      deserializeValue = _ref7.deserializeValue;
  return (0, _streamToObservable.default)(function () {
    return db.createValueStream({
      gte: minKey,
      lte: maxKey
    });
  }).map(function (value) {
    return deserializeValue(value);
  });
}

;

function createReadAllStorage(_ref8) {
  var db = _ref8.db,
      serializeKey = _ref8.serializeKey,
      serializeKeyString = _ref8.serializeKeyString,
      minKey = _ref8.minKey,
      maxKey = _ref8.maxKey,
      deserializeValue = _ref8.deserializeValue;
  var readStorage = createReadStorage({
    db: db,
    serializeKey: serializeKey,
    serializeKeyString: serializeKeyString,
    deserializeValue: deserializeValue
  });
  return {
    get: readStorage.get,
    tryGet: readStorage.tryGet,
    all: createAll({
      db: db,
      minKey: minKey,
      maxKey: maxKey,
      deserializeValue: deserializeValue
    })
  };
}

;

function createReadGetAllStorage(_ref9) {
  var db = _ref9.db,
      serializeKey = _ref9.serializeKey,
      serializeKeyString = _ref9.serializeKeyString,
      getMinKey = _ref9.getMinKey,
      getMaxKey = _ref9.getMaxKey,
      deserializeValue = _ref9.deserializeValue;
  var readStorage = createReadStorage({
    db: db,
    serializeKey: serializeKey,
    serializeKeyString: serializeKeyString,
    deserializeValue: deserializeValue
  });
  return {
    get: readStorage.get,
    tryGet: readStorage.tryGet,
    getAll: function getAll(keys) {
      return createAll({
        db: db,
        minKey: getMinKey(keys),
        maxKey: getMaxKey(keys),
        deserializeValue: deserializeValue
      });
    }
  };
}

;